﻿Imports System.Drawing

Public Class mook_GraphCore
    Public Sub New()
        InitializeComponent()
        LoadDefaultValues()
        InitChart()
    End Sub

    Dim PixelsPer As Integer
    Dim LineDifference As Integer
    Dim ValueMultiplier As Single
    Dim Maximum, Minimum As Single
    Dim AboveColor, UnderColor, GridColor, ChartBackColor, AxesColor As Color

    Dim g As Graphics
    Dim Values As Single()
    Dim CurrentYGridStart As Integer
    Dim CurrentNumberOfValues As Integer
    Dim currentSize As Size = New Size(0, 0)

    Private Sub LoadDefaultValues()
        g = plChart.CreateGraphics()
        PixelsPer = 10
        ChartBackColor = Color.Black
        GridColor = Color.Green
        AboveColor = Color.Chartreuse
        UnderColor = Color.Red
        AxesColor = Color.White
        CurrentYGridStart = 0
        ValueMultiplier = 1
        Maximum = plChart.Size.Height / 2
        Minimum = (-1) * (plChart.Size.Height / 2)
        LineDifference = 1

        Values = New Single(plChart.Size.Width - 1) {}

        For i As Integer = 0 To Values.Length - 1
            Values(i) = 0
        Next
        CurrentNumberOfValues = 0
    End Sub

    Public Sub RefreshControl()
        PostInitChart()
        DrawChart()
    End Sub

    Public Sub AddValue(ByVal val As Single)
        If (Minimum <> 0) And (Maximum <> 0) Then
            If (val * ValueMultiplier > Maximum) Or (val * ValueMultiplier < Minimum) Then
                Return
            End If
        End If
        For i As Integer = 0 To Values.Length - 2
            Values(i) = Values(i + 1)
        Next
        Values(Values.Length - 1) = val * ValueMultiplier
        If (CurrentNumberOfValues < Values.Length) Then
            CurrentNumberOfValues += 1
        End If
        If CurrentYGridStart < (PixelsPer * LineDifference - 1) Then
            CurrentYGridStart += 1
        Else
            CurrentYGridStart = 0
        End If
        DrawChart()
    End Sub

    Private Sub InitChart()
        CurrentYGridStart = 0
        PostInitChart()
    End Sub

    Private Sub plChart_Paint(sender As Object, e As Windows.Forms.PaintEventArgs) Handles plChart.Paint
        If Me.plChart IsNot Nothing Then
            OnResize(New EventArgs())
        End If
    End Sub

    Private Sub PostInitChart()
        If (plChart.Height <> 0) And (plChart.Width <> 0) Then
            g.Clear(ChartBackColor)
            DrawGrid()
        End If
    End Sub

    Private Sub DrawGrid()
        For i As Integer = (plChart.Size.Height / 2) + (PixelsPer * LineDifference) To plChart.Size.Height - 1 Step PixelsPer * LineDifference
            g.DrawLine(New Pen(GridColor), 0, i, plChart.Size.Width, i)
        Next
        For i As Integer = (plChart.Size.Height / 2) - (PixelsPer * LineDifference) To 1 Step -(PixelsPer * LineDifference)
            g.DrawLine(New Pen(GridColor), 0, i, plChart.Size.Width, i)
        Next
        For i As Integer = CurrentYGridStart To plChart.Size.Width - 1 Step PixelsPer * LineDifference
            g.DrawLine(New Pen(GridColor), i, 0, i, plChart.Size.Height)
        Next
        g.DrawLine(New Pen(AxesColor), 0, CInt(plChart.Size.Height / 2), plChart.Size.Width, CInt(plChart.Size.Height / 2))
    End Sub

    Private Sub DrawChart()
        PostInitChart()
        Dim AbovePen As Pen = New Pen(AboveColor)
        Dim UnderPen As Pen = New Pen(UnderColor)

        For i As Integer = Values.Length - CurrentNumberOfValues To Values.Length - 1
            If Values(i) >= 0 Then
                g.DrawLine(AbovePen, Values.Length - i - 1, CInt(plChart.Size.Height / 2) - 1, Values.Length - i - 1, CInt(plChart.Size.Height / 2) - Values(i))
            End If
            If Values(i) < 0 Then
                g.DrawLine(UnderPen, Values.Length - i - 1, CInt(plChart.Size.Height / 2) + 1, Values.Length - i - 1, CInt(plChart.Size.Height / 2) - Values(i))
            End If
        Next

        UnderPen.Dispose()
        AbovePen.Dispose()
    End Sub

    Private Sub RecalculateSize()
        If (currentSize.Height <> 0) And (currentSize.Width <> 0) Then
            Maximum = plChart.Size.Height / 2
            Minimum = (-1) * (plChart.Size.Height / 2)
            Dim SizeChange As Single = CSng(Size.Height) / CSng(currentSize.Height)
            If Size.Height <> 0 Then
                ValueMultiplier += SizeChange
            End If
            Dim i, j As Integer
            Dim NewValues As Single() = New Single(Size.Width - 1) {}
            If Values.Length <= NewValues.Length Then
                j = NewValues.Length - 1
                For i = Values.Length - 1 To 0 Step -1
                    If SizeChange <> 0 Then
                        NewValues(j) = Values(i) * SizeChange
                    End If
                    j -= 1
                Next
            Else
                j = Values.Length - 1
                For i = NewValues.Length - 1 To 0 Step -1
                    If SizeChange <> 0 Then
                        NewValues(i) = Values(j) * SizeChange
                    End If
                    j -= 1
                Next
            End If
            Values = NewValues
            g.Dispose()
            g = plChart.CreateGraphics()
            DrawChart()
        End If
    End Sub

    Protected Overrides Sub OnResize(e As EventArgs)
        MyBase.OnResize(e)
        If plChart IsNot Nothing Then
            If (Size.Height = 0) Or (Size.Width = 0) Then
                Return
            End If
            If (currentSize.Height = 0) And (currentSize.Width = 0) Then
                currentSize = Size
                Return
            End If
            RecalculateSize()
            currentSize = Size
        End If
    End Sub
End Class
