﻿Imports QuartzTypeLib

Public Enum Player_Status
    None
    Stopped
    Paused
    Running
End Enum


Public Class MP3Play
    Private Filtergraph As FilgraphManager  '재생을 위한 경로 및 준비 멤버 개체
    Private FiltergraphTemp As FilgraphManager  '재생 길이를 구하기 위한 경로 및 준비 멤버 개체
    Private MediaControl As IMediaControl  '음악 재생, 정지, 일시 정지 제어 멤버 개체
    Private MediaPosition As IMediaPosition  '음악 재생을 위한 재생 시점 제어 멤버 개체
    Private MediaPositionTemp As IMediaPosition  '재생 길이를 구하기 위한 멤버 개체
    Private BasicAudio As IBasicAudio  '음악 재생 음량, 좌우 발란스 제어 멤버 개체
    Public playStatus = Player_Status.None  '음악 재생 상태 열거형

    Public Function OpenFile(strFilePath As String) As String
        Dim strFileLen = ""
        FiltergraphTemp = New FilgraphManager()
        FiltergraphTemp.RenderFile(strFilePath)

        MediaPositionTemp = CType(FiltergraphTemp, IMediaPosition)
        strFileLen = Convert_Position_To_Time(Convert.ToInt32(MediaPositionTemp.Duration))
        If Not (MediaPositionTemp Is Nothing) Then
            MediaPositionTemp = Nothing
        End If
        If Not (FiltergraphTemp Is Nothing) Then
            FiltergraphTemp = Nothing
        End If
        Return strFileLen
    End Function

    Public Function Convert_Position_To_Time(iPostion As Integer)
        Dim strTimeLen = ""
        Dim iHour As Integer = iPostion / 3600
        Dim iMinute As Integer = Int((iPostion - (iHour * 3600)) / 60)
        Dim iSecond As Integer = Int(iPostion - (iHour * 3600 + iMinute * 60))
        strTimeLen = String.Format("{0:D2}:{1:D2}:{2:D2}", iHour, iMinute, iSecond)

        Return strTimeLen
    End Function

    Public Sub Play(strMP3FilePath As String, setPosition As Double)
        Try
            Filtergraph = New FilgraphManager()
            Filtergraph.RenderFile(strMP3FilePath)
            MediaPosition = CType(Filtergraph, IMediaPosition)
            MediaPosition.CurrentPosition = setPosition
            MediaControl = CType(Filtergraph, IMediaControl)
            playStatus = Player_Status.Running
            MediaControl.Run()
        Catch ex As Exception
        End Try
    End Sub

    Public Sub PlayStop()
        If MediaControl Is Nothing Then Return
        MediaControl.Stop()
        playStatus = Player_Status.Stopped
    End Sub

    Public Sub PlayPause()
        If MediaControl Is Nothing Then Return
        MediaControl.Pause()
        playStatus = Player_Status.Paused
    End Sub

    Public Function Get_CurPosition()
        If playStatus <> Player_Status.Running Then '음악 파일이 재생되지 않을 때
            Return 0
        Else
            Return MediaPosition.CurrentPosition '현재 재생 포지션 반환
        End If
    End Function

    Public Sub Set_CurPosition(trackCurPosition As Double)
        If playStatus = Player_Status.Running Or playStatus = Player_Status.Paused Then '트랙 바 위치와 재생 시점을 동기화 함
            MediaPosition.CurrentPosition = trackCurPosition
        Else
            Return '음악 파일이 재생되지 않을 때
        End If
    End Sub

    Public Function Get_TotalDuration() As Double
        If playStatus = Player_Status.Running Or playStatus = Player_Status.Paused Then '음악 파일 재생 길이
            Return MediaPosition.Duration
        Else
            Return 0 '음악 파일 재생 길이
        End If
    End Function

    Public Sub Set_AudioVolume(vol As Long)
        BasicAudio = Filtergraph
        BasicAudio.Volume = vol '볼륨 설정
    End Sub

    Public Sub Set_AudioBalance(vol As Long)
        BasicAudio = Filtergraph
        BasicAudio.Balance = vol '좌우 밸런스 설정
    End Sub

End Class
